logmode +addin
include ".\biprobitml.prg"
include ".\tablemake.prg"
include ".\hessianmaker.prg"

!debug = 0	
if !debug = 1 then
	logmode + debug
endif
setmaxerrs 10000
%latent1= ""	'1st latent equation specification
%latent2 = ""	'2nd latent equation specification

%smpl = @pagesmpl	  		'sample over which to estimate
%origsmpl = @pagesmpl	'get original sample string
!dogui = 1		'use dialog
!start = 1			'starting value for optimmization
!hill = 1			'hill-climing method
!doanal = 0		'use analytic derivatives for ML estimation
!hessian = 1 	'use Hessian for reporting SEs
%start = """Univariate probit MLE"" ""User-supplied"""
%hill = "Marquart Berndt-Hall-Hall-Hausman"
%hessian = """Observed information matrix"" ""EViews-supplied"""
'parse arguments
if @len(%args) > 0 then
	!pos = @instr(%args," @ ")
	'e.g. biprobit(options) latent1_spec @ latent2_spec
	%latent1 = @left(%args,!pos)	
	%latent2 = @mid(%args,!pos+3)		
	if @len(%latent1) > 0 and @len(%latent2) > 0 then
		!dogui = 0
	endif	
endif
'parse options
if @len(@option(1)) > 0 then
	!start = @hasoption("st")+1
	!hill = @hasoption("b") +1
	!doanal = @hasoption("d")
	!hessian = @hasoption("h") +1
endif
'GUI-------------------------------------------
if !dogui = 1 then
	!return = @uidialog("caption", "Bivariate Probit Model", _
	"edit", %latent1, "First equation (enter a dependent variable followed by a list of regressors)", 400,  _
	"edit", %latent2, "Second equation (enter a dependent variable followed by a list of regressors)",  400, _
	"edit",%smpl,"Enter an estimation sample",  400, _
	"colbreak", _
	"radio", !hessian, "Covariance matrix in SE(s) calculation",%hessian, _
	"radio",!start, "Optimization staring values (optional)",%start, _ 
	"radio", !hill, "Optimization algorithm (optional)",%hill, _
	"check", !doanal, "Use analytic derivatives in ML")
 	if !return = -1 then
 		stop
 	endif
endif

 'check inputted equations are non-null
 if @wcount(%latent1) < 2 then
 	seterr "Invalid  latent equation"
 endif
 if @wcount(%latent2) < 2 then
 	seterr "Invalid response equation"
 endif
  
'Univariate probit estimation for the following reasons
'(1) starting values (2) LR test on RHO (3) error checking on validity of equation specifications.
%start1 = @getnextname("_st1")
%start2 = @getnextname("_st2")
%resid1 = @getnextname("_resid1")
%resid2 = @getnextname("_resid2")
equation {%start1}.probit {%latent1}
{%start1}.makeresid {%resid1}
equation {%start2}.probit {%latent2}	
{%start2}.makeresid {%resid2}

if @wfind(%smpl, "if")>0 then
	%smpl1 =  %smpl + " and "+ %resid1 + "<>na and " +%resid2+ "<>na"	
else 
	%smpl1 = %smpl + " if " + %resid1 + "<>na and " +%resid2+ "<>na"	
endif
if @errorcount>1 then
	@uiprompt("Invalid equation specification")
	smpl {%origsmpl}
	stop
endif	

 'set sample to specified estimation sample
smpl {%smpl1}
   
!lat1len = @wcount(%latent1)-1	'# of coefficient for eq1 
!lat2len = @wcount(%latent2)-1	'# of coefficient for eq2  
'make temporary matrices for storing coefficients
%bn = @getnextname("_B")
%cn = @getnextname("_C")
%rhon = @getnextname("_rho")
%covn = @getnextname("_cov")
%hn = @getnextname("_H")
%alln = @getnextname("_allcoefs")
%tn = @getnextname("_TABLE")

coef(!lat1len) {%bn}
coef(!lat2len) {%cn}
coef(1) {%rhon}
matrix(1,1) {%covn}
matrix(1,1) {%hn}

'univariate probit MLEs for starting values
if !start then
{%bn} = {%start1}.@coef
{%cn} = {%start2}.@coef
endif
'user-supplied starting values
if !start=2 then
	for !j=1 to !lat1len
		{%bn}(!j) = c(!j) 		
	next
	for !t=!lat1len+1 to !lat1len+!lat2len
		{%cn}(!t-!lat1len) = c(!t) 			
	next
	!temprow = !lat1len+!lat2len+1
	{%rhon} = c(!temprow)  
endif

'OPTIMIZATION----------------------------------------
!loglik = 1
!aic = 1
!sc =1
!hq = 1
call ml(%latent1, %latent2, {%bn}, {%cn}, {%rhon}, {%covn}, !loglik, !aic, !hq, !sc, !start, !hill, !doanal)
if @errorcount>1 then
	@uiprompt("Unable to converge, try different starting values")
	d(noerr) {%tn} {%bn} {%cn} {%rhon} {%alln} {%covn} {%hn} {%start1} {%start2} {%resid1} {%resid2}
	smpl {%origsmpl}
	stop
endif	
	
'put coefs into one big coefficient matrix.
!rows = @rows({%bn}) + @rows({%cn}) + 1
coef(!rows) {%alln}
matplace({%alln},{%bn},1,1)
!row = @rows({%bn}) + 1
matplace({%alln},{%cn},!row,1)
!row = !row + @rows({%cn}) 
matplace({%alln},{%rhon},!row,1)

'Do LR test where H0: rho=0
!LR1 = {%start1}.@logl + {%start2}.@logl
!LR = -2*(!LR1-!loglik) 	

'Hessian
%tempgr1 = @getnextname("_tgr1")
%tempgr2 = @getnextname("_tgr2")
group {%tempgr1} {%latent1}
group {%tempgr2} {%latent2}
%obs = @str(@obssmpl)
if !hessian=1 then
	call Hessianmaker({%tempgr1}, {%tempgr2}, {%bn}, {%cn}, {%rhon}, {%hn})
		if @errorcount>1 then
			@uiprompt("Unable to get a Hessian matrix - possibly singularity problem")
			d(noerr) {%tn} {%bn} {%cn} {%rhon} {%alln} {%covn} {%hn} {%start1} {%start2} {%resid1} {%resid2} {%tempgr1} {%tempgr2}
			smpl {%origsmpl}
			stop
		endif	
	'make output table
	call tablemake(%tn, %alln, %hn, %smpl, %obs, %latent1, %latent2, !loglik, !aic, !sc, !hq, !LR)
endif
if !hessian=2 then
	!rhov = {%covn}(!lat1len+!lat2len+1,!lat1len+!lat2len+1)
	!rho = {%rhon}(1)
	!temp = !rhov*4/(@acos(-1)*(1+!rho^2))^2
	{%covn}(!lat1len+!lat2len+1,!lat1len+!lat2len+1)=!temp
	call tablemake(%tn, %alln, %covn, %smpl, %obs, %latent1, %latent2, !loglik, !aic, !sc, !hq, !LR)	
endif

'make display spool
%sn = "results"
if @isobject(%sn) then
	%sn = @getnextname(%sn)
endif
spool {%sn}
{%sn}.append {%tn}
{%sn}.options -tree -borders -titles -comments -margins
show {%sn}

smpl {%origsmpl}
d(noerr) {%tn} {%bn} {%cn} {%rhon} {%alln} {%covn} {%hn} {%start1} {%start2} {%resid1} {%resid2}  {%tempgr1}  {%tempgr2}

